require 'omniauth-oauth2'

module OmniAuth
  module Strategies
    class OpenWechat < OmniAuth::Strategies::OAuth2

    	option :name, "open_wechat"

      option :client_options, {
        site:          "https://api.weixin.qq.com",
        authorize_url: "https://open.weixin.qq.com/connect/qrconnect#wechat_redirect",
        token_url:     "/sns/oauth2/access_token",
        token_method:  :get
      }

      option :token_params, {
        :parse          => :json
      }

      uid do
        raw_info['openid']
      end

      info do
        {
          nickname:   raw_info['nickname'],
          sex:        raw_info['sex'],
          province:   raw_info['province'],
          city:       raw_info['city'],
          country:    raw_info['country'],
          headimgurl: raw_info['headimgurl']
        }
      end

      extra do
        {raw_info: raw_info}
      end

      def request_phase
        params = client.auth_code.authorize_params.merge(redirect_uri: callback_url).merge(authorize_params)
        params["appid"] = params.delete("client_id")
        redirect client.authorize_url(params)
      end

      def raw_info
        @uid ||= access_token["openid"]
        access_token.options[:mode] = :query
        access_token.options[:param_name] = 'access_token'
        @raw_info ||= JSON access_token.get("/sns/userinfo", :params => {"openid" => @uid}).parsed
      end

      protected

        def build_access_token
          params = {
              'appid' => client.id,
              'secret' => client.secret,
              'code' => request.params['code'],
              'grant_type' => 'authorization_code',
              :parse => :json
            }
          client.get_token(params)
        end

    end
  end
end